#include "explosions.h"
#include "sound.h"
#include "game.h"
#include "walls.h"
#include "interface.h"
#include "../common/sources/system/filecache.h"
#include "../common/sources/system/cgraphics2d.h"

CExplosions  explosions;

//---------------------------------------------------------------------------------------------
CExplosions::CExplosions()
{

}

//---------------------------------------------------------------------------------------------
void CExplosions::add(EExplosions type, CVec2 mapPos, int duration)
{
    SExplosion  explo;
    explo.type = type;
    explo.mapPos = mapPos;
    explo.timer.set(duration, true);

    mExploList.push_back(explo);

    if (type == eExplo_Resurrect)
        sound->play(mapPos, "sound/Explosion.wav");
}

//---------------------------------------------------------------------------------------------
void CExplosions::update()
{
    std::vector<SExplosion>::iterator   it = mExploList.begin();

    while(it != mExploList.end())
    {
        if (it->timer.update() == true)
            it = mExploList.erase(it);
        else
            it++;
    }
}

//---------------------------------------------------------------------------------------------
void CExplosions::draw(QImage* image, CVec2 mapPos, CVec2 tablePos)
{
    for (size_t i = 0; i < mExploList.size(); ++i)
    {
        SExplosion& explo = mExploList[i];

        if (explo.mapPos == mapPos)
        {
            QImage  exploGfx;
            CVec2   pos;
            float   scale = 1.0;

            if (explo.type == eExplo_Resurrect)
            {
                exploGfx = fileCache.getImage("gfx/3DView/explosions/Fireball.png");
                pos = CVec2(57, 66);
                scale = 110.0 / 145.0;
            }

            const SWallInfos*   tabData = &gWallsInfos[tablePos.y][tablePos.x][eWallSideUp];
            if (tabData->size.x != 0)
            {
                const SWallInfos*   refWall = NULL;
                refWall = &gWallsInfos[3][2][eWallSideUp];


                // calculate the position and scale factor and draw the explosion in a temporary image
                scale *= (float)tabData->size.y / 111.0f;

                QSize   scaledSize = exploGfx.size() * scale;
                pos = pos - refWall->pos;
                pos.x = pos.x * tabData->size.x / refWall->size.x;
                pos.y = pos.y * tabData->size.y / refWall->size.y;

//                if (flip == true)
//                    pos.x = tabData->size.x - pos.x - scaledSize.width();

                pos += tabData->pos;

                QImage  tempImage(scaledSize, QImage::Format_ARGB32);
                tempImage.fill(TRANSPARENT);
                graph2D.drawImageScaled(&tempImage, CVec2(), exploGfx, scale);

                // darken the explosion based on it's distance
                static const float shadowLevels[] = {0.0f, 0.2f, 0.4f};
                float shadow = shadowLevels[WALL_TABLE_HEIGHT - 1 - tablePos.y];

                graph2D.darken(&tempImage, shadow);

                // draw the explosion on the screen.
                graph2D.drawImage(image, pos, tempImage, 0, false, QRect(0, 33, MAINVIEW_WIDTH, MAINVIEW_HEIGHT));
            }
        }
    }
}
